/*---------------------------------------------------------------------------\
| Project:			Generic LV2 PIC18F458 CAN node firmware
|
| File:				main.c
|
| Description:		Mainline routines.
|
| Global Routines:	Main()
|
| Local Routines:	None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <pic18.h>    	// Include for PICC-18
#include "pic18fxx8.h"	// Register definitions for the PIC18F458
#include "project.h"	// Includes for the project (Globals + misc)
#include "main.h"		// Includes for the mainline
#include "ints.h"		// Includes for interrupts
#include "excepts.h"	// Includes for exception handling
#include "ports.h"		// Includes for the port peripherals
#include "timer2.h"		// Includes for the Timer2 peripheral
#include "serial.h"		// Includes for the serial peripheral (UART )
#include "can.h"		// Includes for the CAN peripheral
#include "bob2.h"
#include <string.h>

// Note that we'll also need the linker file "18F458.LKR"
// which should be added to the Project: Edit Project dialog via
// "Add Node". Be SURE to add files in the MCC18 directory (NOT
// from the MPLAB directory).


/* message currently being processed */
static CanMessage_t curMsg;

#define GPSID (0x06 << 5)

enum {
	UTC_YEAR,
	UTC_MONTH,
	UTC_DAY,
	UTC_HOURS,
	UTC_MINUTES,
	UTC_SECONDS,
};

static const char *months[] = {
	"Jan",
	"Feb",
	"Mar",
	"Apr",
	"May",
	"Jun",
	"Jul",
	"Aug",
	"Sep",
	"Oct",
	"Nov",
	"Dec",
};

#define MAX(a,b) ((a)>(b)?(a):(b))

#define PREC 5
#define FLOATFMT "%+1." #PREC "f"
#define DATEFMT "%2$s %3$d %1$d %4$d:%5$d:%6$d"
static char buf[MAX(4 + PREC, 21)];

/*---------------------------------------------------------------------------\
| Function:		main
|
| Purpose:		Run all Mainline code
|
| Discussion:	Doesn't do much, huh?
|
| Expects:		Uh let's see... nothing.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void main (void)
{
	// Initialize everything and its brother
	Initialize();

	while(TRUE)
	{
		if(canMessageAvailable)
		{
			GetCanMessage(&curMsg);

			switch(curMsg.header.all >> 5)
			{
				case GPSID | 0b00010:				//UTC
					process_UTC();
					break;
				case GPSID | 0b00011:				//Latitude and Longitude
					process_LatLong();
					break;
				case GPSID | 0b00100:				//Height
					process_Height();
					break;
				case GPSID | 0b00001:				//lock
					process_Lock();
					break;
			}
		}
	}
}




/*---------------------------------------------------------------------------\
| Function:		Initialize
|
| Purpose:		Initialize everything. Really.
|
| Discussion:	Did I mention initialization?
|
| Expects:		to be called after a RESET - eg, registers reset to known
|				values and and interrupts off.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Initialize (void)
{

	/*
		EVENTUALLY: Check for a reset: If there hasn't been a reset,
		then something has gone horribly wrong. Do a SOFTWARE RESET
		here to put everything in a known state.
	*/

	// Setting up the CAN bus peripheral: We're passing constants
	// to the initialization routine so that we can just set the
	// initialize initialization :) here.

	const CanHeader_t canMaskB0 = 0b0000011111100000;	//Set acceptance mask0
	const CanHeader_t canMaskB1 = 0x3FF;				//Set acceptance mask1
	const CanHeader_t CanFilterB0F0 = 0x3FF;			//Set acceptance mask1
	//compiler complains that this is not a binary number. How was I supposed to set the
	//a bit to don't care? Had const CanHeader_t CanFilterB0F1 = 0bnnnnnn00000;	//Set Can filter one
	const CanHeader_t CanFilterB0F1 = 0b11111100000;	//Set Can filter one
	const CanHeader_t CanFilterB1F2 = 0b00011000001;	//Set Can filter two
	const CanHeader_t CanFilterB1F3 = 0b00011000010;	//Set Can filter three
	const CanHeader_t CanFilterB1F4 = 0b00011000011;	//Set Can filter four
	const CanHeader_t CanFilterB1F5 = 0b00011000100;	//Set Can filter five

	// Get the outputs stabilized FIRST.

	InitializePorts();

	// Turn on serial transmitter and receiver. Turn on receiver interrupts
	// but leave the transmitter interrupts off (we'll turn them on when
	// there's actual data to send over the port). Make both interrupts
	// low priority, and se the bit rate to 9600bps.

	InitializeSerial (	SERIAL_TRANSMIT_ENABLE
						| SERIAL_TRANSMIT_DISABLE_INTS
						| SERIAL_RECEIVE_ENABLE
						| SERIAL_RECEIVE_DISABLE_INTS
						| SERIAL_BITRATE_9600 );

	// Turn on timer2, HP interrupts @ 1KHz

	InitializeTimer2 (	TIMER2_ENABLE
						| TIMER2_ENABLE_INTS
						| TIMER2_1KHZ );

	// Turn on CAN: See const variable declarations above
	// for the values. Note that as soon as we initialize the
	// the bus, we can start to receive messages, so be sure
	// that interrupts are enabled VERY SOON after this call
	// (No more than about a  message length, which at top
	// speed is about 1/22KHz = 45us which at 20MHz is only
	// 250 instructions.

	InitializeCan (	CAN_BITRATE_1MBPS,
					canMaskB0,			// Mask RXB0
					canMaskB1,			// Mask RXB1
					CanFilterB0F0,		// Filter RXB0 F0
					CanFilterB0F1,		// Filter RXB0 F1
					CanFilterB1F2,		// Filter RXB1 F2
					CanFilterB1F3,		// Filter RXB1 F3
					CanFilterB1F4,		// Filter RXB1 F4
					CanFilterB1F5);		// Filter RXB1 F5

	// Initialize variables before start up

	InitializeVariables();

	InitializeBob2();

	// Initialize interrupts

	InitializeInterrupts();

	// DO NO FURTHER INITIALIZATION AFTER INTERRUPTS SINCE WE'RE OFF AND
	// RUNNING ONCE THEY'RE ENABLED.
}





/*---------------------------------------------------------------------------\
| Function:		InitializeVariables
|
| Purpose:		Initialize all mainline variables
|
| Discussion:	Sure.
|
| Expects:		To be called before interrupts enabled, although it's not
|				a requirement.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/


void InitializeVariables (void)
{
	/* Do nothing */;
}

static uint32_t getData32(uint8_t index)
{
	index <<= 2;
	return curMsg.data[index] << 24 |
	       curMsg.data[index + 1] << 16 |
	       curMsg.data[index + 2] << 8 |
	       curMsg.data[index + 3];
}

void process_UTC()
{
/*jamey wins a prize for trying to confuse us all :)*/
#define T(c) (curMsg.data[UTC_ ## c])
	sprintf(buf, DATEFMT,
		T(YEAR), T(MONTH), T(DAY),
		T(HOURS), T(MINUTES), T(SECONDS));
#undef T
	bobII(buf);
}

void process_LatLong()
{
	bobII("lat:");
	sprintf(buf, FLOATFMT, (float)getData32(0) / 100000000.0);
	bobII(buf);

	bobII("long:");
	sprintf(buf, FLOATFMT, (float)getData32(1) / 100000000.0);
	bobII(buf);
}

void process_Height()
{
	bobII("alt:");
	sprintf(buf, FLOATFMT, (float)getData32(0) / 100.0);
	bobII(buf);
}

void process_Lock()
{
	bobII("sats:");
	sprintf(buf, "%d", curMsg.data[0]);
	bobII(buf);
}
